#include "system.h"

#include "fileio.h"
#include "algorithms.h"


//////// Settings macros ////////

Uint32 getPrivateProfile32(char *section, char *key, char *def, char *path)
{
	char buffer[32];

	GetPrivateProfileString(section, key, def, buffer, 32, path);

	return getInteger(buffer, 10);
}

Uint32 getSetting32(HKEY baseKey, const char *path, const char *value)
{
	// Open the key
	HKEY key;			// Handle to a session with a registry key

	RegOpenKey((HKEY)baseKey, path, &key);

	// Get a value
	DWORD buffer;		// Results of transaction will go here
	DWORD buflen = 4;	// Length of the buffer is 4, naturally
	DWORD type;			// Type will contain type of data transfered

	RegQueryValueEx(key, value, NULL, &type, (BYTE*)&buffer, &buflen);

	// Close the key
	RegCloseKey(key);

	// Check data for validity
	if (type == REG_DWORD)
		return buffer;
	else
		return -1;			// -1 = error
}

void setSetting32(HKEY baseKey, const char *path, const char *value, Uint32 buffer)
{
	// Open the key
	HKEY key;			// Handle to a session with a registry key

	RegCreateKey(baseKey, path, &key);

	// Get a value
	RegSetValueEx(key, value, 0, REG_DWORD, (BYTE*)&buffer, 4);

	// Close the key
	RegCloseKey(key);
}

void getServiceString(HKEY baseKey, const char *path, const char *value, char *buffer)
{
	// Open the key
	HKEY key;			// Handle to a session with a registry key

	RegOpenKey(baseKey, path, &key);

	// Get a value
	DWORD buflen = 40;
	DWORD type;			// Type will contain type of data transfered

	RegQueryValueEx(key, value, NULL, &type, (BYTE*)&buffer, &buflen);

	// Close the key
	RegCloseKey(key);
}

void addNewsChecksum(Uint32 Checksum)
{
	// Extract location to write the new checksum
	Uint32 Position = getSetting32(HKEY_CURRENT_USER, "Software\\Virgin\\SubSpace\\News", "Pos");

	String s;
	if (Position < 1000)
		s += "0";
	if (Position < 100)
		s += "0";
	if (Position < 10)
		s += "0";
	s += (int)Position;

	// Write the checksum
	setSetting32(HKEY_CURRENT_USER, "Software\\Virgin\\SubSpace\\News", s, Checksum);

	// Calculate new location to write the next checksum
	++Position;
	if (Position == 10000)
		Position = 0;

	// Write this location to the registry
	setSetting32(HKEY_CURRENT_USER, "Software\\Virgin\\SubSpace\\News", "Pos", Position);
}

void setWindowTitle(char *title)
{
	char fileName[532];
	Sint32 len = GetModuleFileName(GetModuleHandle(NULL), fileName, 532);

	if (!SetWindowText(FindWindow(NULL, fileName), title))
	{
		Sint32 off = -1;

		for (Sint32 i = len - 1; i > 1; --i)
		{
			switch (fileName[i])
			{
			case '/':
			case '\\':
				off = i + 1;
				i = 0;
				break;
			case '.':
				fileName[i] = '\0';
			};
		}

		if (off != -1)
		{
			SetWindowText(FindWindow(NULL, fileName + off), title);
		}
	}
}

bool readDataLines(char *file, void (*callback)(char *line))
{
	fileAccess fa;				// File access
	if (!fa.open(file, true))	// Open read-only
		return false;

	char buffer[128];			// Temporary storage
	Uint16 i = 0;				// Buffer offset

	char c;						// Read character
	bool skip = false;			// Skip this line? (comments, blank, etc)

	do
	{
		switch (c = fa.readNext())
		{
		case '\0':	// End of file
		case '\n':	// Enter
			if (!skip)
			{	// Don't ignore the buffered line
				if (i)
				{	// Ignore blank lines
					buffer[i] = '\0';
					callback(buffer);
				}
			}
			else
			{	// Reset skip for next line
				skip = false;
			}

			i = 0;	// Reset offset after any line

		case '\r':
			break;

		default:
			if (i == 0)
			{	// Filter comments
				skip = !isAlphaNumeric(c);
			}

			if (i < 127)
			{	// No buffer overruns
				if (isPrintable(c))
				{	// Ignore non-printable
					buffer[i++] = c;
				}
			}
		}
	} while (c);

	return true;
}
